package com.bonait.bnframework.business.deviceControl;

import android.os.Handler;
import android.os.Looper;

import com.bonait.bnframework.common.constant.ConfigName;
import com.bonait.bnframework.common.db.mode.BPA_PLCADDRESS;
import com.bonait.bnframework.common.helper.I.IWriteCallBack;
import com.bonait.bnframework.common.modbus.ModbusMaster;
import com.bonait.bnframework.common.utils.ToastUtils;

/**PLC控制类*/
public class PLCControl{

    //region 单例模式
    private static volatile PLCControl instance = null;
    public static PLCControl get() {
        PLCControl manager = instance;
        if (manager == null) {
            synchronized (PLCControl.class) {
                manager = instance;
                if (manager == null) {
                    manager = new PLCControl();
                    instance = manager;
                }
            }
        }
        return manager;
    }
    private PLCControl(){};
    //endregion

    //region 私有变量
    /**设备的从站ID*/
    private final int slaveId=1;
    /**PLC类型是三菱的*/
    private final PLCType plcType=PLCType.MELSEC;
    //endregion


    public void inital(IWriteCallBack callBack) {

        //TODO:确定初始化逻辑再写。
    }


    /**读取PLC
     * @param address 变量的tag，如心跳时间、通道1开关。
     * @param dataType 数据类型的值就行，如读取bool就true或者false。读取浮点数就是0.0f等
     * */
    public <TDataType> Object readPLC(String address,Class<TDataType> dataType) {
        final Object[] results= {null};
        if (ModbusMaster.get().IsConnected && ConfigName.PLC_Address.containsKey(address)) {
            String addr = ConfigName.PLC_Address.get(address).address;
            if (!addr.isEmpty() && getModAddr(addr) != -1) {
                try {
                    String startAddr = String.valueOf(getModAddr(addr));
                    if (Boolean.class.equals(dataType)) {
                        ModbusMaster.get().ReadBool(startAddr, 1, slaveId, val -> {
                            results[0] = val[0];
                        });
                    } else if (Short.class.equals(dataType)) {
                        ModbusMaster.get().ReadShort(startAddr, 1, slaveId, val -> {
                            results[0] = val[0];
                        });
                    } else if (Integer.class.equals(dataType)) {
                        ModbusMaster.get().ReadInt(startAddr, 1, slaveId, val -> {
                            results[0] = val[0];
                        });
                    } else if (Float.class.equals(dataType)) {
                        ModbusMaster.get().ReadFloat(startAddr, 1, slaveId, val -> {
                            results[0] = val[0];
                        });
                        return dataType.cast(0.0f);
                    } else {
                        // 其他类型的处理逻辑
                        return null; // 默认返回 null
                    }
                } catch (Exception ex) {
                    ToastUtils.error("ReadPLC PLC error:"+ex.getMessage());
                }
            }
        }
        return results[0];
    }


    public <TDataType> void writePLC(String address,TDataType value, IWriteCallBack callBack) {

        if (ModbusMaster.get().IsConnected && ConfigName.PLC_Address.containsKey(address)){
            String addr=ConfigName.PLC_Address.get(address).address;
            if (!addr.isEmpty()&&getModAddr(addr)!=-1){
                try{
                    String startAddr=String.valueOf(getModAddr(addr));
                    //String startAddr=address;
                    if (value instanceof Boolean){
                        ModbusMaster.get().WriteBool(startAddr,(boolean)value,slaveId,callBack);
                    } else if (value instanceof boolean[]) {
                        ModbusMaster.get().WriteBool(startAddr,(boolean[])value,slaveId,callBack);
                    } else if (value instanceof Integer) {
                        ModbusMaster.get().WriteInt(startAddr,(int) value,slaveId,callBack);
                    }else if (value instanceof int[]) {
                        ModbusMaster.get().WriteInt(startAddr,(int[]) value,slaveId,callBack);
                    }else if (value instanceof Short) {
                        ModbusMaster.get().WriteShort(startAddr,(short) value,slaveId,callBack);
                    }else if (value instanceof short[]) {
                        ModbusMaster.get().WriteShort(startAddr,(short[])value,slaveId,callBack );
                    }else if (value instanceof Float) {
                        ModbusMaster.get().WriteFloat(startAddr,(float)value,slaveId,callBack );
                    }else if (value instanceof Float[]) {
                        ModbusMaster.get().WriteFloat(startAddr,(float[]) value,slaveId,callBack );
                    }
                }catch (Exception ex){
                    ToastUtils.error("Write PLC error:"+ex.getMessage());
                }

            }
        }
    }


    public int getCleanTemp(){
        final  short[] temp=new short[]{0};
        if (ModbusMaster.get().IsConnected && ConfigName.PLC_Address.containsKey("清洗温度")){
            String addr=ConfigName.PLC_Address.get("清洗温度").address;
            if (!addr.isEmpty()&& getModAddr(addr)!=-1){
                try{
                    String startAddr=String.valueOf(getModAddr(addr));
                    ModbusMaster.get().ReadShort(startAddr,1,slaveId,val->{
                        temp[0]=val[0];
                    });
                }catch (Exception ex){
                    ToastUtils.error("Write PLC error:"+ex.getMessage());
                }
            }
        }
        return (int)temp[0];
    }

    public boolean[] getAllMotorState(){
       final int readLength=24;
       boolean[] states=new boolean[readLength];
       if (ModbusMaster.get().IsConnected){
           int startAddr=getModAddr("D1");
           if (startAddr!=-1){
               ModbusMaster.get()
                       .ReadShort(String.valueOf(startAddr),readLength,slaveId,val->{
                   if (val.length==readLength){
                       for (int i=0;i<readLength;i++){
                           states[i]=val[i]>0?true:false;
                       }
                   }
               });
           }

       }

       return states;
    }
    /**适用于普通仓
     * @param index 仓位编号。
     * @param delayTime 开启时间。
     * */
    public void openMotorAndClose(int index, int delayTime, IWriteCallBack callBack) {
        try {
            openMotor(index, new IWriteCallBack() {
                @Override
                public void onSuccess() {
                    new Handler(Looper.getMainLooper()).postDelayed(()->{
                        closeMotor(index, new IWriteCallBack() {
                            @Override
                            public void onSuccess() {
                                if (callBack != null){
                                    callBack.onSuccess();
                                }
                            }

                            @Override
                            public void onFailure(String ErrorMsg) {
                                if (callBack!=null){
                                    callBack.onFailure(ErrorMsg);
                                }
                            }
                        });
                    },delayTime);
                }
                @Override
                public void onFailure(String ErrorMsg) {
                    if (callBack != null) {
                        callBack.onFailure(ErrorMsg);
                    }
                }
            });
        } catch (Exception ex) {
            ToastUtils.error("OpenMotorAndClose Exception:" + ex.getMessage());
        }
    }

    /**适用于带有反转回收功能的仓
     * @param index 仓位编号。
     * @param delayTime 开启时间。单位为ms。
     * @param reverseTime 反转时间。单位为ms。
     * */
    public void openMotorAndReverse(int index, int delayTime,int reverseTime, IWriteCallBack callBack) {
        try {
            openMotor(index, new IWriteCallBack() {
                @Override
                public void onSuccess() {
                    new Handler(Looper.getMainLooper()).postDelayed(()->{
                        closeMotor(index, new IWriteCallBack() {
                            @Override
                            public void onSuccess() {
                                reverseMotor(index, new IWriteCallBack() {
                                    @Override
                                    public void onSuccess() {
                                        new Handler(Looper.getMainLooper()).postDelayed(()->{
                                            stopRverseMotor(index, new IWriteCallBack() {
                                                @Override
                                                public void onSuccess() {
                                                    if (callBack!=null){
                                                        callBack.onSuccess();
                                                    }
                                                }

                                                @Override
                                                public void onFailure(String ErrorMsg) {
                                                    if (callBack!=null){
                                                        callBack.onFailure(ErrorMsg);
                                                    }
                                                }
                                            });
                                        },reverseTime);
                                    }

                                    @Override
                                    public void onFailure(String ErrorMsg) {
                                        if (callBack!=null){
                                            callBack.onFailure(ErrorMsg);
                                        }
                                    }
                                });
                            }

                            @Override
                            public void onFailure(String ErrorMsg) {
                                if (callBack!=null){
                                    callBack.onFailure(ErrorMsg);
                                }
                            }
                        });
                    },delayTime);
                }
                @Override
                public void onFailure(String ErrorMsg) {
                    if (callBack!=null){
                        callBack.onFailure(ErrorMsg);
                    }
                }
            });
        } catch (Exception ex) {
            ToastUtils.error("OpenMotorAndReverse Exception:" + ex.getMessage());
        }
    }

    /**反转后自动停止
     * @param index 通道序号
     * @param reverseTime 反转时间
     */
    public void reverMotorAndClose(int index,int reverseTime,IWriteCallBack callBack){
        try {
            reverseMotor(index, new IWriteCallBack() {
                @Override
                public void onSuccess() {
                    new Handler(Looper.getMainLooper()).postDelayed(()->{
                        stopRverseMotor(index, new IWriteCallBack() {
                            @Override
                            public void onSuccess() {
                                if (callBack!=null){
                                    callBack.onSuccess();
                                }
                            }

                            @Override
                            public void onFailure(String ErrorMsg) {
                                if (callBack!=null){
                                    callBack.onFailure(ErrorMsg);
                                }
                            }
                        });
                    },reverseTime);
                }
                @Override
                public void onFailure(String ErrorMsg) {
                    if (callBack!=null){
                        callBack.onFailure(ErrorMsg);
                    }
                }
            });
        } catch (Exception ex) {
            ToastUtils.error("ReverMotorAndClose Exception:" + ex.getMessage());
        }
    }

    public void openMotor(int index, IWriteCallBack callBack) {
        String name="通道"+index+"开关";
        try {
            if (ConfigName.getInstance().PLC_Address.containsKey(name)) {
                BPA_PLCADDRESS plcaddress = ConfigName.getInstance().PLC_Address.get(name);
                if (!plcaddress.address.isEmpty() && ModbusMaster.get().IsConnected) {
                    int startAddr = getModAddr(plcaddress.address);
                    if (startAddr!=-1){
                        ModbusMaster.get().WriteShort(String.valueOf(startAddr),(short)1,slaveId,callBack);
                    }
                }
            }
        } catch (Exception ex) {
            ToastUtils.error("OpenMotor Exception:" + ex.getMessage());
        }
    }

    public void reverseMotor(int index,IWriteCallBack callBack){
        String name="通道"+index+"反转";
        try {
            if (ConfigName.getInstance().PLC_Address.containsKey(name)) {
                BPA_PLCADDRESS plcaddress = ConfigName.getInstance().PLC_Address.get(name);
                if (!plcaddress.address.isEmpty() && ModbusMaster.get().IsConnected) {
                    int startAddr = getModAddr(plcaddress.address);
                    if (startAddr!=-1){
                        ModbusMaster.get().WriteShort(String.valueOf(startAddr),(short)1,slaveId,callBack);
                    }
                }
            }
        } catch (Exception ex) {
            ToastUtils.error("OpenMotor Exception:" + ex.getMessage());
        }
    }

    public void stopRverseMotor(int index,IWriteCallBack callBack){
        String name="通道"+index+"反转";
        try {
            if (ConfigName.getInstance().PLC_Address.containsKey(name)) {
                BPA_PLCADDRESS plcaddress = ConfigName.getInstance().PLC_Address.get(name);
                if (!plcaddress.address.isEmpty() && ModbusMaster.get().IsConnected) {
                    int startAddr = getModAddr(plcaddress.address);
                    if (startAddr!=-1){
                        ModbusMaster.get().WriteShort(String.valueOf(startAddr),(short)0,slaveId,callBack);
                    }
                }
            }
        } catch (Exception ex) {
            ToastUtils.error("OpenMotor Exception:" + ex.getMessage());
        }
    }

    public void closeMotor(int index, IWriteCallBack callBack) {
        String name="通道"+index+"开关";
        try {
            if (ConfigName.getInstance().PLC_Address.containsKey(name)) {
                BPA_PLCADDRESS plcaddress = ConfigName.getInstance().PLC_Address.get(name);
                if (!plcaddress.address.isEmpty() && ModbusMaster.get().IsConnected) {
                    int startAddr = getModAddr(plcaddress.address);
                    if (startAddr!=-1){
                        ModbusMaster.get().WriteShort(String.valueOf(startAddr),(short)0,slaveId,callBack);
                    }
                }
            }
        } catch (Exception ex) {
            ToastUtils.error("OpenMotor Exception:" + ex.getMessage());
        }
    }


    /**适用于普通仓（打开后即可关闭，不需要反转）。
     * @param name 仓位名称
     * @param delayTime 打开时间
     * */
    public void openMotorAndClose(String name, int delayTime, IWriteCallBack callBack) {

    }

    /**打开电机并反转，最后关闭
     * @param name 仓位名称。
     * @param delayTime 开启时间。
     * @param reverseTime 反转时间。
     * */
    public void openAndReverse(String name,int delayTime,int reverseTime,IWriteCallBack callBack){

    }

    public void openMotor(String name, IWriteCallBack callBack) {
        writePLC(name,true,callBack);
    }
    public void reverseMotor(String name, IWriteCallBack callBack) {
        writePLC(name,false,callBack);
    }

    public void closeMotor(String name, IWriteCallBack callBack) {

    }


    public void clrHeartbeat(IWriteCallBack callBack) {
        writePLC("心跳时间",(short)0,callBack);
    }


    public boolean isTimeOut() {
        final boolean[] result = {true};
        if (ModbusMaster.get().IsConnected && ConfigName.PLC_Address.containsKey("断线标志")) {
            String addr = ConfigName.PLC_Address.get("断线标志").address;
            try {
                int startAddr=getModAddr(addr);
                if (!addr.isEmpty()&& startAddr!= -1) {
                    ModbusMaster.get().ReadBool(String.valueOf(startAddr), 1, slaveId, val->{
                        result[0]=val[0];
                    });
                }
            } catch (Exception ex) {
                ToastUtils.error("Exception message:" + ex.getMessage());
            }
        }
        return result[0];
    }


    /**适用于三菱PLC，只有X,Y,M,D区*/
    private int getModAddr(String address){
        if (address == null) return -1;
        address = address.toUpperCase().trim();

        try {
            if (address.length() < 2) return Integer.parseInt(address);

            String res = address.substring(1);
            if (res == null) return -1;

            int startAddress=Integer.parseInt(res) ;
            //如果是M区地址。
            if (address.contains("M")) {
                return startAddress + 4000;
            } else if (address.contains("D")) {
                return startAddress;
            }else if(address.contains("X")){
                return Integer.parseInt(res,8);
            }else if(address.contains("Y")){
                return Integer.parseInt(res,8)+300;
            }else{
                return Integer.parseInt(address);
            }
        } catch (Exception e) {
            return -1;
        }
    }
}
